@@ -1,19 +1,26 @@
 use v6;
 use fatal;
 
-unit class Getopt::Long:ver<0.2.0>;
+unit class Getopt::Long:ver<0.3.0>;
 
-role Exceptional is Exception {
-}
-
-class Exception does Exceptional {
-	has $.message;
+class Exception is CORE::Exception {
+	has Str:D $.message is required;
 	method new(Str $message) {
 		return self.bless(:$message);
 	}
 }
 
-our class ValueInvalid does Exceptional {
+class ValueInvalid is CORE::Exception {
+	has Str:D $.format is required;
+	method new(Str $format) {
+		return self.bless(:$format);
+	}
+	method rethrow-with(Str $name) {
+		die Exception.new($!format.sprintf($name));
+	}
+}
+
+class ConverterInvalid is CORE::Exception {
 	has $.format;
 	method new(Str $format) {
 		return self.bless(:$format);
@@ -23,68 +30,32 @@ our class ValueInvalid does Exceptional {
 	}
 }
 
-my sub null-converter(Str:D $value --> Str) {
-	return $value;
-}
-
-my sub int-converter(Str:D $value --> Int) {
-	return $value.Int;
-}
-
-my sub rat-converter(Str:D $value --> Rat) {
-	return $value.Rat;
-}
-
-my sub num-converter(Str:D $value --> Num) {
-	return $value.Num;
-}
-
-my sub real-converter(Str:D $value --> Real) {
-	return $value.Real;
-}
-
-my sub complex-converter(Str:D $value --> Complex) {
-	return $value.Complex;
-}
-
-my sub io-converter(Str:D $value --> IO::Path) {
-	return $value.IO;
-	CATCH { default {
-		die ValueInvalid.new(qq{Can not convert %s argument "$value" a path: $_.message()});
-	}}
-}
-
-my sub datetime-converter(Str:D $value --> DateTime) {
-	return DateTime.new($value);
-
-	CATCH { when X::Temporal {
-		die ValueInvalid.new(qq{Can not convert %s argument "$value" to datetime: $_.message()});
-	}}
-}
-
-my sub date-converter(Str:D $value --> Date) {
-	return Date.new($value);
-
-	CATCH { when X::Temporal {
-		die ValueInvalid.new(qq{Can not convert %s argument "$value" to date: $_.message()});
-	}}
-}
-
-my sub maybe-converter(Str:D $value --> Any) {
-	return val($value);
+my sub convert(Code:D $converter, Str:D $value) {
+	return $converter($value);
+	CATCH {
+		when X::Str::Numeric {
+			die ValueInvalid.new(qq{Cannot convert %s argument "$value" to number: $_.reason()});
+		}
+		when X::Numeric::CannotConvert {
+			die ValueInvalid.new("Cannot convert %s argument $_.source() to {$_.target // $_.target.perl}: $_.reason()");
+		}
+		default {
+			die ValueInvalid.new("Can not convert %s argument \"$value\": {.message}");
+		}
+	}
 }
 
 my role Store {
 	has Str:D $.key is required;
-	has Sub:D $.converter = &null-converter;
+	has Code:D $.converter = *.self;
 	has Junction:D $.constraints = all();
-	method check-constraints($value) {
+	method check-constraints(Any:D $value) {
 		die ValueInvalid.new(qq{Can't accept %s argument "$value" because it fails its constraints}) unless $value ~~ $!constraints;
 	}
-	method store-convert($value, $hash) {
-		self.store-direct($!converter($value), $hash);
+	method store-convert(Str:D $value, Hash:D $hash) {
+		self.store-direct(convert($!converter, $value), $hash);
 	}
-	method store-direct($value, $hash) { ... }
+	method store-direct(Any:D $value, Hash:D $hash) { ... }
 }
 
 my class ScalarStore does Store {
@@ -95,13 +66,13 @@ my class ScalarStore does Store {
 }
 
 my class CountStore does Store {
-	method store-direct(Any:D $value, Hash:D $hash) {
+	method store-direct(Int:D $value, Hash:D $hash) {
 		$hash{$!key} += $value;
 	}
 }
 
 my class ArrayStore does Store {
-	has Any:U $.type = $!converter.returns;
+	has Any:U $.type is required;
 	method store-direct(Any:D $value, Hash:D $hash) {
 		self.check-constraints($value);
 		$hash{$!key} //= $!type === Any ?? Array !! Array[$!type].new;
@@ -110,10 +81,10 @@ my class ArrayStore does Store {
 }
 
 my class HashStore does Store {
-	has Any:U $.type = $!converter.returns;
+	has Any:U $.type is required;
 	method store-convert(Any:D $pair, Hash:D $hash) {
 		my ($key, $value) = $pair.split('=', 2);
-		my $converted-value = $!converter($value);
+		my $converted-value = convert($!converter, $value);
 		self.check-constraints($converted-value);
 		$hash{$!key} //= $!type === Any ?? Hash !! Hash[$!type].new;
 		$hash{$!key}{$key} = $converted-value;
@@ -124,7 +95,6 @@ my class HashStore does Store {
 }
 
 my class Option {
-	has Str:D $.name is required;
 	has Range:D $.arity is required;
 	has Store:D $.store is required;
 	has Any $.default;
@@ -146,40 +116,56 @@ method !positionals {
 }
 
 method !options {
-	return %!options.values;
+	return %!options;
 }
 
 my %store-for = (
 	'%' => HashStore,
 	'@' => ArrayStore,
+	'$' => ScalarStore,
 	''  => ScalarStore,
 );
 
 my sub make-option(@names, Any:U $multi-class, %multi-args, Range $arity, %options-args?, Bool $negatable?) {
-	return flat @names.map: -> $name {
-		my $store = $multi-class.new(|%multi-args, :key(@names[0]));
-		my @options;
-		@options.push: Option.new(:$name, :$store, :$arity, :default, |%options-args);
+	my $store = $multi-class.new(|%multi-args, :key(@names[0]));
+	my %options;
+	for @names -> $name {
+		%options{$name} = Option.new(:$store, :$arity, :default, |%options-args);
 		if $negatable {
-			@options.push: Option.new(:name("no$name"), :$store, :$arity, |%options-args, :!default);
-			@options.push: Option.new(:name("no-$name"), :$store, :$arity, |%options-args, :!default);
+			%options{"no$name"} = Option.new(:$store, :$arity, |%options-args, :!default);
+			%options{"no-$name"} = Option.new(:$store, :$arity, |%options-args, :!default);
 		}
-		@options;
 	}
+	return %options;
 }
 
-my sub converter-for-format(Str:D $format) {
-	state %converter-for-format =
-		i => &int-converter,
-		o => &int-converter, # compatability with p5
-		s => &null-converter,
-		f => &num-converter,
-		r => &rat-converter,
-		c => &complex-converter,
-		p => &io-converter,
-		d => &datetime-converter,
-		a => &date-converter;
-	return %converter-for-format{$format} // die Exception.new("No such format $format");
+my %converter-for-type{Any:U} = (
+	Pair.new(Int,      *.Int),
+	Pair.new(Rat,      *.Rat),
+	Pair.new(Num,      *.Num),
+	Pair.new(Real,     *.Real),
+	Pair.new(Complex,  *.Complex),
+	Pair.new(Str,      *.Str),
+	Pair.new(IO::Path, *.IO),
+	Pair.new(IO,       *.IO),
+	Pair.new(DateTime, *.DateTime),
+	Pair.new(Date,     *.Date),
+	Pair.new(Any,      &val),
+);
+
+my sub type-for-format(Str:D $format) {
+	state %type-for-format =
+		i => Int,
+		o => Int, # compatability with p5
+		s => Str,
+		f => Num,
+		r => Rat,
+		c => Complex,
+		p => IO::Path,
+		d => DateTime,
+		a => Date;
+	die ConverterInvalid.new("No such format $format for %s") if not %type-for-format{$format}:exists;
+	return %type-for-format{$format};
 };
 
 my rule name { [\w+]+ % '-' | '?' }
@@ -209,17 +195,20 @@ my grammar Argument {
 
 	token counter {
 		'+'
-		{ make [ CountStore, {}, 0..0 ] }
+		{ make [ CountStore, {}, 0..0, { :1default } ] }
 	}
 
 	token type {
 		<alpha>
-		{ make converter-for-format(~$/) }
+		{
+			my $type = type-for-format(~$/);
+			make { :$type, :converter(%converter-for-type{$type}) }
+		}
 	}
 
 	token equals {
 		'=' <type> $<repeat>=[<[%@]>?]
-		{ make [ %store-for{~$<repeat>}, { :converter($<type>.ast) }, 1..1 ] }
+		{ make [ %store-for{~$<repeat>}, $<type>.ast, 1..1 ] }
 	}
 
 	rule range {
@@ -228,93 +217,88 @@ my grammar Argument {
 	}
 	token equals-more {
 		'=' <type> '{' <range>'}'
-		{ make [ ArrayStore, { :converter($<type>.ast) }, $<range>.ast ] }
+		{ make [ ArrayStore, $<type>.ast, $<range>.ast ] }
 	}
 
 	token colon-type {
 		':' <type>
-		{ make [ ScalarStore, { :converter($<type>.ast) }, 0..1, { :default($<type>.ast.returns.new) } ] }
+		{ make [ ScalarStore, $<type>.ast, 0..1, { :default($<type>.ast<type>.new) } ] }
 	}
 
 	token colon-int {
 		':' $<num>=[<[0..9]>+]
-		{ make [ ScalarStore, { :converter(&int-converter) }, 0..1, { :default($<num>.Int) } ] }
+		{ make [ ScalarStore, { :converter(*.Int) }, 0..1, { :default($<num>.Int) } ] }
 	}
 
 	token colon-count {
 		':+'
-		{ make [ CountStore, { :converter(&int-converter) }, 0..1, { :default(1) } ] }
+		{ make [ CountStore, { :converter(*.Int) }, 0..1, { :default(1) } ] }
 	}
 }
 
-method new-from-patterns(Getopt::Long:U: @patterns, Str :$positionals = "") {
+method new-from-patterns(Getopt::Long:U: @patterns, Str:D :$positionals = "") {
 	my %options;
 	for @patterns -> $pattern {
 		if Argument.parse($pattern) -> $match {
-			for @($match.ast) -> $option {
-				%options{$option.name} = $option;
+			for $match.ast.kv -> $key, $option {
+				%options{$key} = $option;
 			}
 		}
 		else {
-			die Exception.new("Couldn't parse '$pattern'");
+			die Exception.new("Couldn't parse argument specification '$pattern'");
 		}
+		CATCH { when ConverterInvalid {
+			.rethrow-with("pattern $pattern");
+		}}
 	}
-	my @positionals = $positionals.comb.map(&converter-for-format);
+	my @positionals = $positionals.comb.map(&type-for-format).map(&get-converter);
 	return self.new(:%options, :@positionals);
 }
 
-our role Parseable {
-	method parse-argument(Parseable:U: Str $value) { ... }
-}
-
-sub get-converter(Any:U $type) {
-	state %converter-for-type{Any:U} = (
-		(Int)      => &int-converter,
-		(Rat)      => &rat-converter,
-		(Num)      => &num-converter,
-		(Real)     => &real-converter,
-		(Complex)  => &complex-converter,
-		(Str)      => &null-converter,
-		(IO::Path) => &io-converter,
-		(DateTime) => &datetime-converter,
-		(Date)     => &date-converter,
-		(Any)      => &maybe-converter,
-	);
-
+my sub get-converter(Any:U $type) {
+	state $coercion-how = try ::("Metamodel::CoercionHOW");
 	if %converter-for-type{$type} -> &converter {
 		return &converter;
 	}
+	elsif $type.HOW ~~ $coercion-how {
+		my &primary = get-converter($type.^constraint_type());
+		return sub coercion-converter(Str $input) {
+			return $type.^coerce(primary($input));
+		}
+	}
 	elsif $type.HOW ~~ Metamodel::EnumHOW {
 		my $valid-values = $type.WHO.keys.sort({ $type.WHO{$^value} }).join(", ");
-		sub enum-converter(Str $value) {
+		return sub enum-converter(Str $value) {
 			return $type.WHO{$value} // die ValueInvalid.new(qq{Can't convert %s argument "$value" to $type.^name(), valid values are: $valid-values});
 		}
-		trait_mod:<returns>(&enum-converter, $type);
-		return &enum-converter;
-	}
-	elsif $type ~~ Parseable {
-		my $callback = sub (Str $value) { return $type.parse-argument($value) };
-		trait_mod:<returns>($callback, $type);
-		return $callback;
 	}
 	else {
-		die Exception.new("No conversion known for type {$type.^name}");
+		die ConverterInvalid.new("No argument conversion known for %s argument (type {$type.^name})");
 	}
 }
 
 my role Formatted {
-	has Positional $.format is required;
+	has Option %.options is required;
 }
 
-multi sub trait_mod:<is>(Parameter $param, Str:D :$getopt!) is export(:DEFAULT, :traits) {
-	with Argument.parse($getopt, :rule('argument')) -> $match {
-		return $param does Formatted(:format($match.ast));
+multi sub trait_mod:<is>(Parameter $param, Str:D :getopt(:$option)!) is export(:DEFAULT, :traits) {
+	CATCH { when ConverterInvalid { .rethrow("parameter {$param.name}") }}
+	with Argument.parse($option, :rule('argument')) -> $match {
+		my %options = make-option($param.named_names, |$match.ast);
+		return $param does Formatted(:%options);
 	}
 	else {
-		die Exception.new("Couldn't parse '$getopt'");
+		die Exception.new("Couldn't parse parameter {$param.name}'s argument specification '$option'");
 	}
 }
 
+multi sub trait_mod:<is>(Parameter $param, Code:D :option($converter)!) is export(:DEFAULT, :traits) {
+	my $element-type = $param.sigil eq '@'|'%' ?? $param.type.of !! $param.type;
+	my $type = $element-type ~~ Any ?? $element-type !! Any;
+	my %options = make-option($param.named_names, %store-for{$param.sigil}, { :$type, :$converter }, 1..1);
+	return $param does Formatted(:%options);
+}
+
 my role Parsed {
 	has Getopt::Long:D $.getopt is required;
 }
@@ -323,22 +307,22 @@ multi sub trait_mod:<is>(Sub $sub, :$getopt!) is export(:DEFAULT, :traits) {
 	$sub does Parsed(Getopt::Long.new-from-sub($sub));
 }
 
-my multi get-positionals($candidate) {
-	return $candidate.signature.params.grep(*.positional).map(*.type);
+my multi get-positionals(&candidate) {
+	return &candidate.signature.params.grep(*.positional).map(*.type);
 }
 
-my multi get-positionals(Parsed $candidate) {
-	return $candidate.getopt!positionals;
+my multi get-positionals(&candidate where Parsed) {
+	return &candidate.getopt!positionals;
 }
 
-my multi get-named($candidate) {
+my multi get-named(&candidate) {
 	my @options;
-	for $candidate.signature.params.grep(*.named) -> $param {
-		my @names = $param.named_names;
+	for &candidate.signature.params.grep(*.named) -> $param {
 		if $param ~~ Formatted {
-			@options.append: make-option(@names, |$param.format.list);
+			@options.append: $param.options;
 		}
 		else {
+			my @names = $param.named_names;
 			if $param.sigil eq '$' {
 				my $type = $param.type;
 				my $constraints = $param.constraints;
@@ -355,22 +339,27 @@ my multi get-named($candidate) {
 				my $converter = get-converter($type);
 				@options.append: make-option(@names, %store-for{$param.sigil}, { :$type, :$converter }, 1..1);
 			}
+			CATCH { when ConverterInvalid {
+				.rethrow-with("parameter {$param.name}");
+			}}
 		}
 	}
-	return @options;
+	return @options.hash;
 }
-my multi get-named(Parsed $candidate) {
-	return $candidate.getopt!options;
+my multi get-named(&candidate where Parsed) {
+	return &candidate.getopt!options;
 }
 
+my Str @ordinals = <first second third fourth fifth sixth seventh eighth nineth tenth some some> ... *;
+
 method new-from-sub(Getopt::Long:U: Sub $main) {
 	my (%options, @positional-types);
 	for $main.candidates -> $candidate {
-		for get-named($candidate) -> $option {
-			if %options{$option.name}:exists and %options{$option.name} !eqv $option {
-				die Exception.new("Can't merge arguments for {$option.name}");
+		for get-named($candidate).kv -> $key, $option {
+			if %options{$key}:exists and %options{$key} !eqv $option {
+				die Exception.new("Can't merge arguments for {$key}");
 			}
-			%options{$option.name} = $option;
+			%options{$key} = $option;
 		}
 		@positional-types.push: get-positionals($candidate);
 	}
@@ -378,105 +367,85 @@ method new-from-sub(Getopt::Long:U: Sub $main) {
 	my @positionals = (0 ..^ $elem-max).map: -> $index {
 		my @types = @positional-types.grep(* > $index)»[$index];
 		die Exception.new("Positional arguments are of different types {@types.perl}") unless [===] @types;
+		CATCH { when ConverterInvalid {
+			.rethrow-with(@ordinals[$index]);
+		}}
 		get-converter(@types[0]);
 	}
 	return self.new(:%options, :@positionals);
 }
 
-method get-options(Getopt::Long:D: @args is copy, :%hash, :$auto-abbreviate = False, :$compat-builtin = False, :named-anywhere(:$permute) = !$compat-builtin, :$bundling = !$compat-builtin, :$compat-singles = $compat-builtin, :$compat-negation = $compat-builtin, :$compat-positional = $compat-builtin, :$write-args) {
+method get-options(Getopt::Long:D: @args is copy, :%hash, :$auto-abbreviate = False, :$compat-builtin = False, :named-anywhere(:$permute) = !$compat-builtin, :$bundling = !$compat-builtin, :$compat-singles = $compat-builtin, :$compat-negation = $compat-builtin, :$compat-positional = $compat-builtin, :$auto-help = $compat-builtin, :$write-args) {
 	my @list;
 
-	sub wrap-exceptions(Str $description, Str $value, &action) {
-		return action($value);
-		CATCH {
-			when ValueInvalid {
-				.rethrow-with($description);
-			}
-			when X::Str::Numeric {
-				$_ does role :: does Exceptional {
-					method message() {
-						qq{Cannot convert $description argument "$value" to number: $.reason};
-					}
-				}
-				.rethrow;
-			}
-			when X::Numeric::CannotConvert {
-				$_ does role :: does Exceptional {
-					method message() {
-						"Cannot convert $description argument $.source to {$.target // $.target.perl}: $.reason"
-					}
-				}
-				.rethrow;
-			}
-		}
-	}
-
 	while @args {
 		my $head = @args.shift;
 
 		my $consumed = 0;
 
-		sub get-option($key) {
+		sub get-option(Str:D $key, Str:D $name) {
 			with %!options{$key} -> $option {
 				return $option;
 			}
+			elsif $key eq 'help' && $auto-help {
+				return Option.new(:store(ScalarStore.new(:key<help>)), :arity(0..0), :default);
+			}
 			elsif $auto-abbreviate {
 				my @names = %!options.keys.grep(*.starts-with($key));
 				if @names == 1 {
 					return %!options{ @names[0] };
 				}
 				elsif @names > 1 {
-					die Exception.new("Ambiguous partial option '$key', possible interpretations: @names[]");
+					die Exception.new("Ambiguous partial option $name, possible interpretations: @names[]");
 				}
 				else {
-					die Exception.new("Unknown option $key");
+					die Exception.new("Unknown option $name");
 				}
 			}
 			else {
-				die Exception.new("Unknown option $key");
+				die Exception.new("Unknown option $name");
 			}
 		}
 
-		sub take-value($option, $value) {
-			wrap-exceptions("--$option.name()", $value, -> $value {
-				$option.store($value, %hash);
-				$consumed++;
-			});
+		sub take-value(Option:D $option, Str:D $value, Str:D $name) {
+			CATCH { when ValueInvalid { .rethrow-with($name) } }
+			$option.store($value, %hash);
+			$consumed++;
 		}
-		sub take-args($option) {
+		sub take-args(Option:D $option, Str:D $name) {
 			while @args && $consumed < $option.arity.min {
-				take-value($option, @args.shift);
+				take-value($option, @args.shift, $name);
 			}
 
 			while @args && $consumed < $option.arity.max && !@args[0].starts-with('--') {
-				take-value($option, @args.shift);
+				take-value($option, @args.shift, $name);
 			}
 
 			if $consumed == 0 && $option.arity.min == 0 {
 				$option.store-default(%hash);
 			}
 			elsif $consumed < $option.arity.min {
-				die Exception.new("No argument given for option {$option.name}");
+				die Exception.new("The argument $name requires a value but none was specified");
 			}
 		}
 
 		my rule name { [\w+]+ % '-' | '?' }
 
 		if $compat-singles && $head ~~ / ^ '-' <name> '=' $<value>=[.*] / -> $/ {
-			my $option = get-option($<name>);
-			die Exception.new("$<name> doesn't take one argument") if $option.arity.max != 1;
-			take-value($option, ~$<value>);
+			my $option = get-option(~$<name>, "-$<name>");
+			die Exception.new("-$<name> doesn't take an argument") if $option.arity.max != 1;
+			take-value($option, ~$<value>, "-$<name>");
 		}
 		elsif $bundling && $head ~~ / ^ '-' $<values>=[\w .* ] $ / -> $/ {
 			my @values = $<values>.Str.comb;
 			for @values.keys -> $index {
 				my $value = @values[$index];
-				my $option = get-option($value);
+				my $option = get-option($value, "-$value");
 				if $option.arity.max > 0 && $index + 1 < @values.elems {
-					take-value($option, $<values>.substr($index + 1));
+					take-value($option, $<values>.substr($index + 1), "-$value");
 				}
 
-				take-args($option);
+				take-args($option, $value);
 				last if $consumed;
 			}
 		}
@@ -485,22 +454,22 @@ method get-options(Getopt::Long:D: @args is copy, :%hash, :$auto-abbreviate = Fa
 			last;
 		}
 		elsif $head ~~ / ^ '-' ** 1..2 <name> $ / -> $/ {
-			take-args(get-option($<name>));
+			take-args(get-option(~$<name>, ~$/), ~$/);
 		}
-		elsif $head ~~ / ^ '--' <name> '=' $<value>=[.*] / -> $/ {
-			my $option = get-option($<name>);
-			die Exception.new("$<name> doesn't take arguments") if $option.arity.max == 0;
-			take-value($option, ~$<value>);
-			take-args($option);
+		elsif $head ~~ / ^ $<full-name>=[ '--' <name> ] '=' $<value>=[.*] / -> $/ {
+			my $option = get-option(~$<name>, ~$<full-name>);
+			die Exception.new("Option $<full-name> doesn't take arguments") if $option.arity.max == 0;
+			take-value($option, ~$<value>, ~$<full-name>);
+			take-args($option, ~$<full-name>);
 		}
-		elsif $compat-negation && $head ~~ / ^ '-' ** 1..2 '/' <name> ['=' $<value>=[.*]]?  $ / {
+		elsif $compat-negation && $head ~~ / ^ $<full-name>=[ '-' ** 1..2 '/' <name> ] ['=' $<value>=[.*]]?  $ / {
 			if $<value> {
-				my $option = get-option($<name>);
-				die Exception.new("$<name> doesn't take an argument") if $option.arity.max != 1;
-				take-value($option, ~$<value> but False);
+				my $option = get-option(~$<name>, ~$<full-name>);
+				die Exception.new("Option $<full-name> doesn't take an argument") if $option.arity.max != 1;
+				take-value($option, ~$<value> but False, ~$<full-name>);
 			}
 			else {
-				take-args(get-option('no-' ~ $<name>));
+				take-args(get-option('no-' ~ $<name>, ~$<full-name>), ~$<full-name>);
 			}
 		}
 		else {
@@ -514,11 +483,13 @@ method get-options(Getopt::Long:D: @args is copy, :%hash, :$auto-abbreviate = Fa
 		}
 	}
 	@$write-args = @list if $write-args;
-	state Str @labels = <first second third fourth fifth sixth seventh eighth nineth tenth some some> ... *;
-	my &fallback-converter = $compat-positional ?? &maybe-converter !! &null-converter;
+	my &fallback-converter = $compat-positional ?? &val !! *.self;
 	my @converters = |@!positionals, &fallback-converter, *;
-	@list = (@labels Z @list Z @converters).map: { wrap-exceptions(|@^args) }
-	return \(|@list, |%hash);
+	my @positionals = (@ordinals Z @list Z @converters).map: -> $ [ $name, $value, $converter ] {
+		CATCH { when ValueInvalid { .rethrow-with($name) }}
+		convert($converter, $value);
+	};
+	return \(|@positionals, |%hash);
 }
 
 our sub get-options-from(@args, *@elements, :$overwrite, *%config) is export(:DEFAULT, :functions) {
@@ -554,22 +525,22 @@ our sub get-options(|args) is export(:DEFAULT, :functions) {
 	return get-options-from(@*ARGS, :overwrite, |args);
 }
 
-our sub call-with-getopt(&func, @args, %options?, :$overwrite) is export(:DEFAULT, :functions) {
-	my $capture = Getopt::Long.new-from-sub(&func).get-options(@args, |%options, :write-args($overwrite ?? @args !! Any));
+our sub call-with-getopt(&func, @args, %options?) is export(:DEFAULT, :functions) {
+	my $capture = Getopt::Long.new-from-sub(&func).get-options(@args, |%options, :write-args(@args));
 	return func(|$capture);
 }
 
-my sub call-main(CallFrame $callframe, $retval) {
+my sub call-main(CallFrame $callframe, Any $retval) {
 	my $main = $callframe.my<&MAIN>;
 	return $retval unless $main;
 	my %options = %*SUB-MAIN-OPTS // {};
-	return call-with-getopt($main, @*ARGS, %options, :overwrite);
+	return call-with-getopt($main, @*ARGS, %options);
 }
 
 our sub ARGS-TO-CAPTURE(Sub $func, @args) is export(:DEFAULT, :MAIN) {
 	my %options = %*SUB-MAIN-OPTS // {};
 	return Getopt::Long.new-from-sub($func).get-options(@args, |%options, :write-args(@args));
-	CATCH { when Exceptional { note .message; &*EXIT(2) } };
+	CATCH { when Exception { note .message; &*EXIT(2) } };
 }
 
 our &MAIN_HELPER is export(:DEFAULT, :MAIN) = $*PERL.compiler.version after 2018.06
@@ -690,16 +661,12 @@ It supports the following types for named and positional arguments:
 =item DateTime
 =item Date
 
-It also supports any enum type, as well as any type that implements
-the Getopt::Long::Parseable role, e.g.:
+It also supports any enum type, and any coercion type that uses any of
+the aforementioned types as its contraint type (e.g. C<Foo(Str)>).
 
-class FourtyTwo does Parseable {
-  has Int $.value;
-  method parse-argument(Int(Str) $value) {
-    die Getopt::Long::Invalid("Argument %s is not 42") if $value != 42;
-    return FourtyTwo.new(:$value);
-  }
-}
+An explicit converter can also be set using an `is option` trait, e.g.
+
+ sub MAIN(Foo :$foo is option(&foo-converter)) { ... }
 
 =head2 Simple options
 
@@ -756,7 +723,7 @@ C<< $verbose >> by setting its value to C<False>.
 An incremental option is specified with a plus C<+> after the
 option name:
 
-    sub MAIN(Int :$verbose is getopt('+')) { ... }
+    sub MAIN(Int :$verbose is option('+')) { ... }
 
 or:
 
@@ -858,8 +825,8 @@ For example, the above command line would be handled as follows:
 
 or
 
-    sub MAIN(Rat :@coordinates is getopt('f{2}'),
-      Int :@rgbcolor is getopt('i{3}'))
+    sub MAIN(Rat :@coordinates is option('f{2}'),
+      Int :@rgbcolor is option('i{3}'))
 
 
     get-options('coordinates=f{2}' => my @coordinates,
@@ -1116,8 +1083,8 @@ compat-builtin (default: C<False>)
 
 Enable all compatibility options that make argument parsing more like
 the builtin argument parsing. Currently that means disabling C<bundling>
-and C<permute>, and enabling C<compat-singles>, C<compat-negation> and
-C<compat-positional>.
+and C<permute>, and enabling C<compat-singles>, C<compat-negation>,
+C<compat-positional> and C<auto-help>)
 
 =end item
 
@@ -1183,10 +1150,18 @@ possible.
 
 =end item
 
+=begin item
+auto-help (default: C<$compat-builtin>)
+
+This adds an extra --help option, that can hook into Raku's built-in
+usage message generator.
+
+=end item
+
 =head1 Return values and Errors
 
 C<get-options> returns a capture to indicate success, or throws an
-C<Getopt::Long::Exceptional> otherwise.
+C<Getopt::Long::Exception> otherwise.
 
 =head1 Troubleshooting
 
